@@ -121,7 +121,11 @@ _numba_rnd_random_seed(rnd_state_t *state)
 {
     PyObject *timemod, *timeobj;
     double timeval;
-    unsigned int seed, rshift;
+    Py_uintptr_t seed;
+    unsigned int seed32;
+    void *dummy;
+
+    /* XXX we could get a seed using _PyOS_URandom() instead */
 
     timemod = PyImport_ImportModuleNoBlock("time");
     if (!timemod)
@@ -133,18 +137,21 @@ _numba_rnd_random_seed(rnd_state_t *state)
     if (timeval == -1 && PyErr_Occurred())
         return -1;
     /* Mix in seconds and nanoseconds */
-    seed = (unsigned) timeval ^ (unsigned) (timeval * 1e9);
+    seed = (Py_uintptr_t) timeval ^ (Py_uintptr_t) (timeval * 1e9);
 #ifndef _WIN32
     seed ^= getpid();
 #endif
-    /* Address space randomization bits: take MSBs of various pointers.
-     * It is counter-productive to shift by 32, since the virtual address
-     * space width is usually less than 64 bits (48 on x86-64).
-     */
-    rshift = sizeof(void *) > 4 ? 16 : 0;
-    seed ^= (Py_uintptr_t) &timemod >> rshift;
-    seed += (Py_uintptr_t) &PyObject_CallMethod >> rshift;
-    numba_rnd_init(state, seed);
+    /* Address space randomization bits: mix in various pointers. */
+    seed ^= (Py_uintptr_t) &timemod;
+    seed += (Py_uintptr_t) &PyObject_CallMethod >> 3;
+    seed += (Py_uintptr_t) &rnd_init_by_array;
+    dummy = malloc(1);
+    free(dummy);
+    seed += (Py_uintptr_t) &dummy;
+
+    /* Reduce to 32 bits for Mersenne Twisted seeding */
+    seed32 = (unsigned int) (seed ^ (seed >> 16));
+    numba_rnd_init(state, seed32);
     return 0;
 }
 
@@ -376,7 +383,7 @@ numba_poisson_ptrs(rnd_state_t *state, double lam)
             continue;
         }
         if ((log(V) + log(invalpha) - log(a/(us*us)+b)) <=
-            (-lam + k*loglam - loggam(k+1)))
+            (-lam + (double) k*loglam - loggam((double) k+1)))
         {
             return k;
         }
@@ -873,325 +880,6 @@ import_cython_function(const char *module_name, const char *function_name)
     return res;
 }
 
-/*
- * BLAS calling helpers.  The helpers can be called without the GIL held.
- * The caller is responsible for checking arguments (especially dimensions).
- */
-
-
-/* Fast getters caching the value of a function's address after
-   the first call to import_cblas_function(). */
-
-#define EMIT_GET_CBLAS_FUNC(name)                                 \
-    static void *cblas_ ## name = NULL;                           \
-    static void *get_cblas_ ## name(void) {                       \
-        if (cblas_ ## name == NULL) {                             \
-            PyGILState_STATE st = PyGILState_Ensure();            \
-            const char *mod = "scipy.linalg.cython_blas";         \
-            cblas_ ## name = import_cython_function(mod, # name); \
-            PyGILState_Release(st);                               \
-        }                                                         \
-        return cblas_ ## name;                                    \
-    }
-
-EMIT_GET_CBLAS_FUNC(dgemm)
-EMIT_GET_CBLAS_FUNC(sgemm)
-EMIT_GET_CBLAS_FUNC(cgemm)
-EMIT_GET_CBLAS_FUNC(zgemm)
-EMIT_GET_CBLAS_FUNC(dgemv)
-EMIT_GET_CBLAS_FUNC(sgemv)
-EMIT_GET_CBLAS_FUNC(cgemv)
-EMIT_GET_CBLAS_FUNC(zgemv)
-EMIT_GET_CBLAS_FUNC(ddot)
-EMIT_GET_CBLAS_FUNC(sdot)
-EMIT_GET_CBLAS_FUNC(cdotu)
-EMIT_GET_CBLAS_FUNC(zdotu)
-EMIT_GET_CBLAS_FUNC(cdotc)
-EMIT_GET_CBLAS_FUNC(zdotc)
-
-#undef EMIT_GET_CBLAS_FUNC
-
-typedef float (*sdot_t)(int *n, void *dx, int *incx, void *dy, int *incy);
-typedef double (*ddot_t)(int *n, void *dx, int *incx, void *dy, int *incy);
-typedef npy_complex64 (*cdot_t)(int *n, void *dx, int *incx, void *dy, int *incy);
-typedef npy_complex128 (*zdot_t)(int *n, void *dx, int *incx, void *dy, int *incy);
-
-typedef void (*xxgemv_t)(char *trans, int *m, int *n,
-                         void *alpha, void *a, int *lda,
-                         void *x, int *incx, void *beta,
-                         void *y, int *incy);
-
-typedef void (*xxgemm_t)(char *transa, char *transb,
-                         int *m, int *n, int *k,
-                         void *alpha, void *a, int *lda,
-                         void *b, int *ldb, void *beta,
-                         void *c, int *ldc);
-
-/* Vector * vector: result = dx * dy */
-NUMBA_EXPORT_FUNC(int)
-numba_xxdot(char kind, char conjugate, Py_ssize_t n, void *dx, void *dy,
-            void *result)
-{
-    void *raw_func = NULL;
-    int _n;
-    int inc = 1;
-
-    switch (kind) {
-        case 'd':
-            raw_func = get_cblas_ddot();
-            break;
-        case 's':
-            raw_func = get_cblas_sdot();
-            break;
-        case 'c':
-            raw_func = conjugate ? get_cblas_cdotc() : get_cblas_cdotu();
-            break;
-        case 'z':
-            raw_func = conjugate ? get_cblas_zdotc() : get_cblas_zdotu();
-            break;
-        default:
-            {
-                PyGILState_STATE st = PyGILState_Ensure();
-                PyErr_SetString(PyExc_ValueError,
-                                "invalid kind of *DOT function");
-                PyGILState_Release(st);
-            }
-            return -1;
-    }
-    if (raw_func == NULL)
-        return -1;
-
-    _n = (int) n;
-
-    switch (kind) {
-        case 'd':
-            *(double *) result = (*(ddot_t) raw_func)(&_n, dx, &inc, dy, &inc);;
-            break;
-        case 's':
-            *(float *) result = (*(sdot_t) raw_func)(&_n, dx, &inc, dy, &inc);;
-            break;
-        case 'c':
-            *(npy_complex64 *) result = (*(cdot_t) raw_func)(&_n, dx, &inc, dy, &inc);;
-            break;
-        case 'z':
-            *(npy_complex128 *) result = (*(zdot_t) raw_func)(&_n, dx, &inc, dy, &inc);;
-            break;
-    }
-
-    return 0;
-}
-
-/* Matrix * vector: y = alpha * a * x + beta * y */
-NUMBA_EXPORT_FUNC(int)
-numba_xxgemv(char kind, char *trans, Py_ssize_t m, Py_ssize_t n,
-             void *alpha, void *a, Py_ssize_t lda,
-             void *x, void *beta, void *y)
-{
-    void *raw_func = NULL;
-    int _m, _n;
-    int _lda;
-    int inc = 1;
-
-    switch (kind) {
-        case 'd':
-            raw_func = get_cblas_dgemv();
-            break;
-        case 's':
-            raw_func = get_cblas_sgemv();
-            break;
-        case 'c':
-            raw_func = get_cblas_cgemv();
-            break;
-        case 'z':
-            raw_func = get_cblas_zgemv();
-            break;
-        default:
-            {
-                PyGILState_STATE st = PyGILState_Ensure();
-                PyErr_SetString(PyExc_ValueError,
-                                "invalid kind of *GEMV function");
-                PyGILState_Release(st);
-            }
-            return -1;
-    }
-    if (raw_func == NULL)
-        return -1;
-
-    _m = (int) m;
-    _n = (int) n;
-    _lda = (int) lda;
-
-    (*(xxgemv_t) raw_func)(trans, &_m, &_n, alpha, a, &_lda,
-                           x, &inc, beta, y, &inc);
-    return 0;
-}
-
-/* Matrix * matrix: c = alpha * a * b + beta * c */
-NUMBA_EXPORT_FUNC(int)
-numba_xxgemm(char kind, char *transa, char *transb,
-             Py_ssize_t m, Py_ssize_t n, Py_ssize_t k,
-             void *alpha, void *a, Py_ssize_t lda,
-             void *b, Py_ssize_t ldb, void *beta,
-             void *c, Py_ssize_t ldc)
-{
-    void *raw_func = NULL;
-    int _m, _n, _k;
-    int _lda, _ldb, _ldc;
-
-    switch (kind) {
-        case 'd':
-            raw_func = get_cblas_dgemm();
-            break;
-        case 's':
-            raw_func = get_cblas_sgemm();
-            break;
-        case 'c':
-            raw_func = get_cblas_cgemm();
-            break;
-        case 'z':
-            raw_func = get_cblas_zgemm();
-            break;
-        default:
-            {
-                PyGILState_STATE st = PyGILState_Ensure();
-                PyErr_SetString(PyExc_ValueError,
-                                "invalid kind of *GEMM function");
-                PyGILState_Release(st);
-            }
-            return -1;
-    }
-    if (raw_func == NULL)
-        return -1;
-
-    _m = (int) m;
-    _n = (int) n;
-    _k = (int) k;
-    _lda = (int) lda;
-    _ldb = (int) ldb;
-    _ldc = (int) ldc;
-
-    (*(xxgemm_t) raw_func)(transa, transb, &_m, &_n, &_k, alpha, a, &_lda,
-                           b, &_ldb, beta, c, &_ldc);
-    return 0;
-}
-
-/*
- * LAPACK calling helpers.  The helpers can be called without the GIL held.
- * The caller is responsible for checking arguments (especially dimensions).
- */
-
-/* Fast getters caching the value of a function's address after
-   the first call to import_clapack_function(). */
-
-#define EMIT_GET_CLAPACK_FUNC(name)                                 \
-    static void *clapack_ ## name = NULL;                           \
-    static void *get_clapack_ ## name(void) {                       \
-        if (clapack_ ## name == NULL) {                             \
-            PyGILState_STATE st = PyGILState_Ensure();              \
-            const char *mod = "scipy.linalg.cython_lapack";         \
-            clapack_ ## name = import_cython_function(mod, # name); \
-            PyGILState_Release(st);                                 \
-        }                                                           \
-        return clapack_ ## name;                                    \
-    }
-
-EMIT_GET_CLAPACK_FUNC(sgetrf)
-EMIT_GET_CLAPACK_FUNC(dgetrf)
-EMIT_GET_CLAPACK_FUNC(cgetrf)
-EMIT_GET_CLAPACK_FUNC(zgetrf)
-EMIT_GET_CLAPACK_FUNC(sgetri)
-EMIT_GET_CLAPACK_FUNC(dgetri)
-EMIT_GET_CLAPACK_FUNC(cgetri)
-EMIT_GET_CLAPACK_FUNC(zgetri)
-
-
-#undef EMIT_GET_CLAPACK_FUNC
-
-typedef void (*xxgetrf_t)(int *m, int *n, void *a, int *lda, int *ipiv, int *info);
-
-typedef void (*xxgetri_t)(int *n, void *a, int *lda, int *ipiv, void *work, int *lwork, int*info);
-
-/* Compute LU decomposition of a*/
-NUMBA_EXPORT_FUNC(int)
-numba_xxgetrf(char kind, Py_ssize_t m, Py_ssize_t n, void *a, Py_ssize_t lda, int *ipiv, int *info)  /* XXX should these be int * or Pyssize_t */
-{
-    void *raw_func = NULL;
-    int _m, _n, _lda;
-
-    switch (kind) {
-        case 's':
-            raw_func = get_clapack_sgetrf();
-            break;
-        case 'd':
-            raw_func = get_clapack_dgetrf();
-            break;
-        case 'c':
-            raw_func = get_clapack_cgetrf();
-            break;
-        case 'z':
-            raw_func = get_clapack_zgetrf();
-            break;
-        default:
-            {
-                PyGILState_STATE st = PyGILState_Ensure();
-                PyErr_SetString(PyExc_ValueError,
-                                "invalid kind of *LU factorization function");
-                PyGILState_Release(st);
-            }
-            return -1;
-    }
-    if (raw_func == NULL)
-        return -1;
-
-    _m = (int) m;
-    _n = (int) n;
-    _lda = (int) lda;
-
-    (*(xxgetrf_t) raw_func)(&_m, &_n, a, &_lda, ipiv, info);
-    return 0;
-}
-
-
-/* Compute the inverse of a  matrix given its LU decomposition*/
-NUMBA_EXPORT_FUNC(int)
-numba_xxgetri(char kind, Py_ssize_t n, void *a, Py_ssize_t lda, int *ipiv, void *work, int *lwork, int *info) /* XXX int * */
-{
-    void *raw_func = NULL;
-    int _n, _lda;
-
-    switch (kind) {
-        case 's':
-            raw_func = get_clapack_sgetri();
-            break;
-        case 'd':
-            raw_func = get_clapack_dgetri();
-            break;
-        case 'c':
-            raw_func = get_clapack_cgetri();
-            break;
-        case 'z':
-            raw_func = get_clapack_zgetri();
-            break;
-        default:
-            {
-                PyGILState_STATE st = PyGILState_Ensure();
-                PyErr_SetString(PyExc_ValueError,
-                                "invalid kind of *inversion from LU factorization function");
-                PyGILState_Release(st);
-            }
-            return -1;
-    }
-    if (raw_func == NULL)
-        return -1;
-
-    _n = (int) n;
-    _lda = (int) lda;
-
-    (*(xxgetri_t) raw_func)(&_n, a, &_lda, ipiv, work, lwork, info);
-    return 0;
-}
-
-
 /* We use separate functions for datetime64 and timedelta64, to ensure
  * proper type checking.
  */
@@ -1422,6 +1110,20 @@ numba_unpack_slice(PyObject *obj,
 #undef FETCH_MEMBER
 }
 
+NUMBA_EXPORT_FUNC(int)
+numba_fatal_error(void)
+{
+    PyGILState_Ensure();
+#if PY_MAJOR_VERSION < 3
+    /* Py_FatalError doesn't print the current error on Python 2, do it
+       ourselves. */
+    if (PyErr_Occurred())
+        PyErr_Print();
+#endif
+    Py_FatalError("in Numba-compiled function");
+    return 0; /* unreachable */
+}
+
 /* Logic for raising an arbitrary object.  Adapted from CPython's ceval.c.
    This *consumes* a reference count to its argument. */
 NUMBA_EXPORT_FUNC(int)
@@ -1554,3 +1256,9 @@ Define bridge for all math functions
 
 #undef MATH_UNARY
 #undef MATH_BINARY
+
+/*
+ * BLAS and LAPACK wrappers
+ */
+
+#include "_lapack.c"
