@@ -64,6 +64,7 @@ build_c_helpers_dict(void)
     declmethod(fptouif);
     declmethod(gil_ensure);
     declmethod(gil_release);
+    declmethod(fatal_error);
     declmethod(py_type);
     declmethod(unpack_slice);
     declmethod(do_raise);
@@ -83,6 +84,12 @@ build_c_helpers_dict(void)
     declmethod(xxdot);
     declmethod(xxgetrf);
     declmethod(xxgetri);
+    declmethod(xxpotrf);
+    declmethod(ez_rgeev);
+    declmethod(ez_cgeev);
+    declmethod(ez_gesdd);
+    declmethod(ez_geqrf);
+    declmethod(ez_xxgqr);
 
     declpointer(py_random_state);
     declpointer(np_random_state);
@@ -116,6 +123,7 @@ static PyMethodDef ext_methods[] = {
 PyAPI_FUNC(double) _numba_test_sin(double x);
 PyAPI_FUNC(double) _numba_test_cos(double x);
 PyAPI_FUNC(double) _numba_test_exp(double x);
+PyAPI_FUNC(void) _numba_test_vsquare(int n, double *x, double *out);
 
 double _numba_test_sin(double x)
 {
@@ -132,6 +140,20 @@ double _numba_test_exp(double x)
     return exp(x);
 }
 
+void _numba_test_vsquare(int n, double *x, double *out)
+{
+    int i;
+    for (i = 0; i < n; i++)
+        out[i] = pow(x[i], 2.0);
+}
+
+void _numba_test_vcube(int n, double *x, double *out)
+{
+    int i;
+    for (i = 0; i < n; i++)
+        out[i] = pow(x[i], 3.0);
+}
+
 
 MOD_INIT(_helperlib) {
     PyObject *m;
