@@ -9,10 +9,11 @@ from . import config, sigutils
 from .errors import DeprecationError
 from .targets import registry
 
+
+
 # -----------------------------------------------------------------------------
 # Decorators
 
-
 def autojit(*args, **kws):
     """Deprecated.
 
@@ -30,6 +31,7 @@ class _DisableJitWrapper(object):
     def __call__(self, *args, **kwargs):
         return self.py_func(*args, **kwargs)
 
+
 _msg_deprecated_signature_arg = ("Deprecated keyword argument `{0}`. "
                                  "Signatures should be passed as the first "
                                  "positional argument.")
@@ -200,3 +202,27 @@ def njit(*args, **kws):
         warnings.warn('forceobj is set for njit and is ignored', RuntimeWarning)
     kws.update({'nopython': True})
     return jit(*args, **kws)
+
+
+def cfunc(sig, locals={}, cache=False, **options):
+    """
+    This decorator is used to compile a Python function into a C callback
+    usable with foreign C libraries.
+
+    Usage::
+        @cfunc("float64(float64, float64)", nopython=True, cache=True)
+        def add(a, b):
+            return a + b
+
+    """
+    sig = sigutils.normalize_signature(sig)
+
+    def wrapper(func):
+        from .ccallback import CFunc
+        res = CFunc(func, sig, locals=locals, options=options)
+        if cache:
+            res.enable_caching()
+        res.compile()
+        return res
+
+    return wrapper
