@@ -555,6 +555,7 @@ class Interpreter(object):
                 raise e
 
     def dump(self, file=None):
+        # Avoid early bind of sys.stdout as default value
         file = file or sys.stdout
         for offset, block in sorted(self.blocks.items()):
             print('label %s:' % (offset,), file=file)
@@ -630,13 +631,6 @@ class Interpreter(object):
         self.current_block_offset = offset
         return blk
 
-    def block_constains_opname(self, offset, opname):
-        for offset in self.cfa.blocks[offset]:
-            inst = self.bytecode[offset]
-            if inst.opname == opname:
-                return True
-        return False
-
     # --- Bytecode handlers ---
 
     def op_PRINT_ITEM(self, inst, item, printvar, res):
