@@ -4,8 +4,9 @@ from collections import namedtuple, defaultdict
 import copy
 import os
 import sys
+from itertools import permutations, takewhile
 
-import numpy
+import numpy as np
 
 from llvmlite import ir as llvmir
 import llvmlite.llvmpy.core as lc
@@ -15,10 +16,10 @@ import llvmlite.binding as ll
 from numba import types, utils, cgutils, typing
 from numba import _dynfunc, _helperlib
 from numba.pythonapi import PythonAPI
-from . import arrayobj, builtins, optional, imputils
+from . import arrayobj, builtins, imputils
 from .imputils import (user_function, user_generator,
-                                    builtin_registry, impl_ret_borrowed,
-                                    RegistryLoader)
+                       builtin_registry, impl_ret_borrowed,
+                       RegistryLoader)
 from numba import datamodel
 
 
@@ -34,29 +35,85 @@ class OverloadSelector(object):
 
     In the current implementation:
     - a "signature" is a tuple of type classes or type instances
-    - the "best candidate" is simply the first match
+    - the "best candidate" is the most specific match
     """
 
     def __init__(self):
         # A list of (formal args tuple, value)
         self.versions = []
+        self._cache = {}
 
     def find(self, sig):
+        out = self._cache.get(sig)
+        if out is None:
+            out = self._find(sig)
+            self._cache[sig] = out
+        return out
+
+    def _find(self, sig):
+        candidates = self._select_compatible(sig)
+        if candidates:
+            return candidates[self._best_signature(candidates)]
+        else:
+            raise NotImplementedError(self, sig)
+
+    def _select_compatible(self, sig):
+        """
+        Select all compatible signatures and their implementation.
+        """
+        out = {}
         for ver_sig, impl in self.versions:
-            if ver_sig == sig:
-                return impl
-
-            # As generic type
             if self._match_arglist(ver_sig, sig):
-                return impl
+                out[ver_sig] = impl
+        return out
 
-        raise NotImplementedError(self, sig)
+    def _best_signature(self, candidates):
+        """
+        Returns the best signature out of the candidates
+        """
+        ordered, genericity = self._sort_signatures(candidates)
+        # check for ambiguous signatures
+        if len(ordered) > 1:
+            firstscore = genericity[ordered[0]]
+            same = list(takewhile(lambda x: genericity[x] == firstscore,
+                                  ordered))
+            if len(same) > 1:
+                msg = ["{n} ambiguous signatures".format(n=len(same))]
+                for sig in same:
+                    msg += ["{0} => {1}".format(sig, candidates[sig])]
+                raise TypeError('\n'.join(msg))
+        return ordered[0]
+
+    def _sort_signatures(self, candidates):
+        """
+        Sort signatures in ascending level of genericity.
+
+        Returns a 2-tuple:
+
+            * ordered list of signatures
+            * dictionary containing genericity scores
+        """
+        # score by genericity
+        genericity = defaultdict(int)
+        for this, other in permutations(candidates.keys(), r=2):
+            matched = self._match_arglist(formal_args=this, actual_args=other)
+            if matched:
+                # genericity score +1 for every another compatible signature
+                genericity[this] += 1
+        # order candidates in ascending level of genericity
+        ordered = sorted(candidates.keys(), key=lambda x: genericity[x])
+        return ordered, genericity
 
     def _match_arglist(self, formal_args, actual_args):
+        """
+        Returns True if the the signature is "matching".
+        A formal signature is "matching" if the actual signature matches exactly
+        or if the formal signature is a compatible generic signature.
+        """
+        # normalize VarArg
         if formal_args and isinstance(formal_args[-1], types.VarArg):
-            formal_args = (
-                formal_args[:-1] +
-                (formal_args[-1].dtype,) * (len(actual_args) - len(formal_args) + 1))
+            ndiff = len(actual_args) - len(formal_args) + 1
+            formal_args = formal_args[:-1] + (formal_args[-1].dtype,) * ndiff
 
         if len(formal_args) != len(actual_args):
             return False
@@ -74,11 +131,13 @@ class OverloadSelector(object):
         elif types.Any == formal:
             # formal argument is any
             return True
-        elif (isinstance(formal, type) and
-              isinstance(actual, formal)):
-            # formal argument is a type class matching actual argument
-            assert issubclass(formal, types.Type)
-            return True
+        elif isinstance(formal, type) and issubclass(formal, types.Type):
+            if isinstance(actual, type) and issubclass(actual, formal):
+                # formal arg is a type class and actual arg is a subclass
+                return True
+            elif isinstance(actual, formal):
+                # formal arg is a type class of which actual arg is an instance
+                return True
 
     def append(self, value, sig):
         """
@@ -86,7 +145,7 @@ class OverloadSelector(object):
         """
         assert isinstance(sig, tuple), (value, sig)
         self.versions.append((sig, value))
-
+        self._cache.clear()
 
 @utils.runonce
 def _load_global_helpers():
@@ -156,6 +215,7 @@ class BaseContext(object):
         self._getattrs = defaultdict(OverloadSelector)
         self._setattrs = defaultdict(OverloadSelector)
         self._casts = OverloadSelector()
+        self._get_constants = OverloadSelector()
         # Other declarations
         self._generators = {}
         self.special_ops = {}
@@ -178,8 +238,8 @@ class BaseContext(object):
         Useful for third-party extensions.
         """
         # Populate built-in registry
-        from . import (arraymath, iterators, linalg, rangeobj, slicing,
-                       smartarray, tupleobj)
+        from . import (arraymath, enumimpl, iterators, linalg, numbers,
+                       optional, rangeobj, slicing, smartarray, tupleobj)
         try:
             from . import npdatetime
         except NotImplementedError:
@@ -198,6 +258,9 @@ class BaseContext(object):
     def get_arg_packer(self, fe_args):
         return datamodel.ArgPacker(self.data_model_manager, fe_args)
 
+    def get_data_packer(self, fe_types):
+        return datamodel.DataPacker(self.data_model_manager, fe_types)
+
     @property
     def target_data(self):
         raise NotImplementedError
@@ -227,6 +290,7 @@ class BaseContext(object):
         self._insert_getattr_defn(loader.new_registrations('getattrs'))
         self._insert_setattr_defn(loader.new_registrations('setattrs'))
         self._insert_cast_defn(loader.new_registrations('casts'))
+        self._insert_get_constant_defn(loader.new_registrations('constants'))
 
     def insert_func_defn(self, defns):
         for impl, func, sig in defns:
@@ -244,6 +308,10 @@ class BaseContext(object):
         for impl, sig in defns:
             self._casts.append(impl, sig)
 
+    def _insert_get_constant_defn(self, defns):
+        for impl, sig in defns:
+            self._get_constants.append(impl, sig)
+
     def insert_user_function(self, func, fndesc, libs=()):
         impl = user_function(fndesc, libs)
         self._defns[func].append(impl, impl.signature)
@@ -350,95 +418,23 @@ class BaseContext(object):
         dm = self.data_model_manager[ty]
         return dm.load_from_data_pointer(builder, ptr, align)
 
-    def is_struct_type(self, ty):
-        return isinstance(self.data_model_manager[ty], datamodel.CompositeModel)
-
     def get_constant_generic(self, builder, ty, val):
         """
         Return a LLVM constant representing value *val* of Numba type *ty*.
         """
-        if isinstance(ty, types.ExternalFunctionPointer):
-            ptrty = self.get_function_pointer_type(ty)
-            ptrval = ty.get_pointer(val)
-            return builder.inttoptr(self.get_constant(types.intp, ptrval),
-                                    ptrty)
-
-        elif isinstance(ty, types.Array):
-            return self.make_constant_array(builder, ty, val)
-
-        elif isinstance(ty, types.Dummy):
-            return self.get_dummy_value()
-
-        elif self.is_struct_type(ty):
-            struct = self.get_constant_struct(builder, ty, val)
-            if isinstance(ty, types.Record):
-                ptrty = self.data_model_manager[ty].get_data_type()
-                ptr = cgutils.alloca_once(builder, ptrty)
-                builder.store(struct, ptr)
-                return ptr
-            return struct
-
-        else:
-            return self.get_constant(ty, val)
-
-    def get_constant_struct(self, builder, ty, val):
-        assert self.is_struct_type(ty)
-
-        if ty in types.complex_domain:
-            if ty == types.complex64:
-                innertype = types.float32
-            elif ty == types.complex128:
-                innertype = types.float64
-            else:
-                raise Exception("unreachable")
-
-            real = self.get_constant(innertype, val.real)
-            imag = self.get_constant(innertype, val.imag)
-            const = Constant.struct([real, imag])
-            return const
-
-        elif isinstance(ty, (types.Tuple, types.NamedTuple)):
-            consts = [self.get_constant_generic(builder, ty.types[i], v)
-                      for i, v in enumerate(val)]
-            return Constant.struct(consts)
-
-        elif isinstance(ty, types.Record):
-            consts = [self.get_constant(types.int8, b)
-                      for b in bytearray(val.tostring())]
-            return Constant.array(consts[0].type, consts)
-
-        else:
-            raise NotImplementedError("%s as constant unsupported" % ty)
+        try:
+            impl = self._get_constants.find((ty,))
+            return impl(self, builder, ty, val)
+        except NotImplementedError:
+            raise NotImplementedError("cannot lower constant of type '%s'" % (ty,))
 
     def get_constant(self, ty, val):
-        assert not self.is_struct_type(ty)
-
-        lty = self.get_value_type(ty)
-
-        if ty == types.none:
-            assert val is None
-            return self.get_dummy_value()
-
-        elif ty == types.boolean:
-            return Constant.int(Type.int(1), int(val))
-
-        elif ty in types.signed_domain:
-            return Constant.int_signextend(lty, val)
-
-        elif ty in types.unsigned_domain:
-            return Constant.int(lty, val)
-
-        elif ty in types.real_domain:
-            return Constant.real(lty, val)
-
-        elif isinstance(ty, (types.NPDatetime, types.NPTimedelta)):
-            return Constant.real(lty, val.astype(numpy.int64))
-
-        elif isinstance(ty, (types.UniTuple, types.NamedUniTuple)):
-            consts = [self.get_constant(ty.dtype, v) for v in val]
-            return Constant.array(consts[0].type, consts)
-
-        raise NotImplementedError("cannot lower constant of type '%s'" % (ty,))
+        """
+        Same as get_constant_generic(), but without specifying *builder*.
+        Works only for simple types.
+        """
+        # HACK: pass builder=None to preserve get_constant() API
+        return self.get_constant_generic(None, ty, val)
 
     def get_constant_undef(self, ty):
         lty = self.get_value_type(ty)
@@ -612,8 +608,8 @@ class BaseContext(object):
             impl = self._casts.find((fromty, toty))
             return impl(self, builder, fromty, toty, val)
         except NotImplementedError:
-            # Re-raise below
-            raise NotImplementedError("cast", val, fromty, toty)
+            raise NotImplementedError(
+                "Cannot cast %s to %s: %s" % (fromty, toty, val))
 
     def generic_compare(self, builder, key, argtypes, args):
         """
@@ -712,10 +708,13 @@ class BaseContext(object):
     def get_dummy_type(self):
         return GENERIC_POINTER
 
-    def compile_only_no_cache(self, builder, impl, sig, locals={}, flags=None):
-        """Invoke the compiler to compile a function to be used inside a
+    def compile_subroutine_no_cache(self, builder, impl, sig, locals={}, flags=None):
+        """
+        Invoke the compiler to compile a function to be used inside a
         nopython function, but without generating code to call that
         function.
+
+        Note this context's flags are not inherited.
         """
         # Compile
         from numba import compiler
@@ -742,15 +741,15 @@ class BaseContext(object):
         Return a placeholder object that's callable from another Numba
         function.
         """
-        cache_key = (impl.__code__, sig)
+        cache_key = (impl.__code__, sig, type(self.error_model))
         if impl.__closure__:
             # XXX This obviously won't work if a cell's value is
             # unhashable.
             cache_key += tuple(c.cell_contents for c in impl.__closure__)
         ty = self.cached_internal_func.get(cache_key)
         if ty is None:
-            cres = self.compile_only_no_cache(builder, impl, sig,
-                                              locals=locals)
+            cres = self.compile_subroutine_no_cache(builder, impl, sig,
+                                                    locals=locals)
             ty = types.NumbaFunction(cres.fndesc, sig)
             self.cached_internal_func[cache_key] = ty
         return ty
@@ -764,7 +763,8 @@ class BaseContext(object):
         return self.call_internal(builder, ty.fndesc, sig, args)
 
     def call_internal(self, builder, fndesc, sig, args):
-        """Given the function descriptor of an internally compiled function,
+        """
+        Given the function descriptor of an internally compiled function,
         emit a call to that function with the given arguments.
         """
         # Add call to the generated function
@@ -857,7 +857,9 @@ class BaseContext(object):
         # Handle data: reify the flattened array in "C" order as a
         # global array of bytes.
         flat = ary.flatten()
-        consts = Constant.array(Type.int(8), bytearray(flat))
+        # Note: we use `bytearray(flat.data)` instead of `bytearray(flat)` to
+        #       workaround issue #1850 which is due to numpy issue #3147
+        consts = Constant.array(Type.int(8), bytearray(flat.data))
         data = cgutils.global_constant(builder, ".const.array.data", consts)
 
         # Handle shape
@@ -866,7 +868,7 @@ class BaseContext(object):
         cshape = Constant.array(llintp, shapevals)
 
         # Handle strides: use strides of the equivalent C-contiguous array.
-        contig = numpy.ascontiguousarray(ary)
+        contig = np.ascontiguousarray(ary)
         stridevals = [self.get_constant(types.intp, s) for s in contig.strides]
         cstrides = Constant.array(llintp, stridevals)
 
