@@ -3,7 +3,6 @@ from __future__ import print_function, absolute_import
 import sys
 
 import llvmlite.llvmpy.core as lc
-import llvmlite.llvmpy.ee as le
 import llvmlite.binding as ll
 
 from numba import _dynfunc, config
@@ -11,7 +10,7 @@ from numba.callwrapper import PyCallWrapper
 from .base import BaseContext, PYOBJECT
 from numba import utils, cgutils, types
 from numba.utils import cached_property
-from numba.targets import callconv, codegen, externals, intrinsics, listobj
+from numba.targets import callconv, codegen, externals, intrinsics, listobj, setobj
 from .options import TargetOptions
 from numba.runtime import rtsys
 
@@ -65,8 +64,9 @@ class CPUContext(BaseContext):
     def target_data(self):
         return self._internal_codegen.target_data
 
-    def with_aot_codegen(self, name):
-        return self.subtarget(_internal_codegen=codegen.AOTCPUCodegen(name),
+    def with_aot_codegen(self, name, **aot_options):
+        aot_codegen = codegen.AOTCPUCodegen(name, **aot_options)
+        return self.subtarget(_internal_codegen=aot_codegen,
                               aot_mode=True)
 
     def codegen(self):
@@ -114,6 +114,12 @@ class CPUContext(BaseContext):
         """
         return listobj.build_list(self, builder, list_type, items)
 
+    def build_set(self, builder, set_type, items):
+        """
+        Build a set from the Numba *set_type* and its initial *items*.
+        """
+        return setobj.build_set(self, builder, set_type, items)
+
     def post_lowering(self, mod, library):
         if self.is32bit:
             # 32-bit machine needs to replace all 64-bit div/rem to avoid
