@@ -17,14 +17,6 @@ def iterator_getiter(context, builder, sig, args):
 #-------------------------------------------------------------------------------
 # builtin `enumerate` implementation
 
-def make_enumerate_cls(enum_type):
-    """
-    Return the Structure representation of the given *enum_type* (an
-    instance of types.EnumerateType).
-    """
-    return cgutils.create_struct_proxy(enum_type)
-
-
 @lower_builtin(enumerate, types.IterableType)
 @lower_builtin(enumerate, types.IterableType, types.Integer)
 def make_enumerate_object(context, builder, sig, args):
@@ -40,8 +32,7 @@ def make_enumerate_object(context, builder, sig, args):
 
     iterobj = call_getiter(context, builder, srcty, src)
 
-    enumcls = make_enumerate_cls(sig.return_type)
-    enum = enumcls(context, builder)
+    enum = context.make_helper(builder, sig.return_type)
 
     countptr = cgutils.alloca_once(builder, start_val.type)
     builder.store(start_val, countptr)
@@ -58,8 +49,7 @@ def iternext_enumerate(context, builder, sig, args, result):
     [enumty] = sig.args
     [enum] = args
 
-    enumcls = make_enumerate_cls(enumty)
-    enum = enumcls(context, builder, value=enum)
+    enum = context.make_helper(builder, enumty, value=enum)
 
     count = builder.load(enum.count)
     ncount = builder.add(count, context.get_constant(types.intp, 1))
@@ -78,21 +68,13 @@ def iternext_enumerate(context, builder, sig, args, result):
 #-------------------------------------------------------------------------------
 # builtin `zip` implementation
 
-def make_zip_cls(zip_type):
-    """
-    Return the Structure representation of the given *zip_type* (an
-    instance of types.ZipType).
-    """
-    return cgutils.create_struct_proxy(zip_type)
-
 @lower_builtin(zip, types.VarArg(types.Any))
 def make_zip_object(context, builder, sig, args):
     zip_type = sig.return_type
 
     assert len(args) == len(zip_type.source_types)
 
-    zipcls = make_zip_cls(zip_type)
-    zipobj = zipcls(context, builder)
+    zipobj = context.make_helper(builder, zip_type)
 
     for i, (arg, srcty) in enumerate(zip(args, sig.args)):
         zipobj[i] = call_getiter(context, builder, srcty, arg)
@@ -106,8 +88,7 @@ def iternext_zip(context, builder, sig, args, result):
     [zip_type] = sig.args
     [zipobj] = args
 
-    zipcls = make_zip_cls(zip_type)
-    zipobj = zipcls(context, builder, value=zipobj)
+    zipobj = context.make_helper(builder, zip_type, value=zipobj)
 
     if len(zipobj) == 0:
         # zip() is an empty iterator
