@@ -8,7 +8,8 @@ import numpy as np
 from numba import unittest_support as unittest
 from numba import jit, typeof, types
 from numba.compiler import compile_isolated
-from numba.numpy_support import as_dtype, strict_ufunc_typing
+from numba.numpy_support import (as_dtype, strict_ufunc_typing,
+                                 version as numpy_version)
 from .support import TestCase, CompilationCache, MemoryLeak, MemoryLeakMixin, tag
 
 
@@ -65,6 +66,12 @@ def array_transpose(arr):
 def array_copy(arr):
     return arr.copy()
 
+def np_copy(arr):
+    return np.copy(arr)
+
+def np_asfortranarray(arr):
+    return np.asfortranarray(arr)
+
 def array_view(arr, newtype):
     return arr.view(newtype)
 
@@ -388,10 +395,17 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
     def test_np_frombuffer_dtype(self):
         self.check_np_frombuffer(np_frombuffer_dtype)
 
-    def check_layout_dependent_func(self, pyfunc, fac=np.arange):
+    def check_layout_dependent_func(self, pyfunc, fac=np.arange,
+                                    check_sameness=True):
+        def is_same(a, b):
+            return a.ctypes.data == b.ctypes.data
         def check_arr(arr):
             cres = compile_isolated(pyfunc, (typeof(arr),))
-            self.assertPreciseEqual(cres.entry_point(arr), pyfunc(arr))
+            expected = pyfunc(arr)
+            got = cres.entry_point(arr)
+            self.assertPreciseEqual(expected, got)
+            if check_sameness:
+                self.assertEqual(is_same(expected, arr), is_same(got, arr))
         arr = fac(24)
         check_arr(arr)
         check_arr(arr.reshape((3, 8)))
@@ -414,6 +428,13 @@ class TestArrayMethods(MemoryLeakMixin, TestCase):
     def test_array_copy(self):
         self.check_layout_dependent_func(array_copy)
 
+    def test_np_copy(self):
+        self.check_layout_dependent_func(np_copy)
+
+    def test_np_asfortranarray(self):
+        self.check_layout_dependent_func(np_asfortranarray,
+                                         check_sameness=numpy_version >= (1, 8))
+
     def check_np_frombuffer_allocated(self, pyfunc):
         def run(shape):
             cres = self.ccache.compile(pyfunc, (typeof(shape),))
