@@ -1,4 +1,4 @@
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import, print_function, division
 
 from collections import OrderedDict
 import ctypes
@@ -7,10 +7,20 @@ import numpy as np
 
 from numba import (float32, float64, int16, int32, boolean, deferred_type,
                    optional)
-from numba import njit, typeof
+from numba import njit, typeof, errors
 from numba import unittest_support as unittest
 from numba import jitclass
 from .support import TestCase, MemoryLeakMixin, tag
+from numba.jitclass import _box
+from numba.runtime.nrt import MemInfo
+from numba.errors import LoweringError
+
+
+def _get_meminfo(box):
+    ptr = _box.box_get_meminfoptr(box)
+    mi = MemInfo(ptr)
+    mi.acquire()
+    return mi
 
 
 class TestJitClass(TestCase, MemoryLeakMixin):
@@ -18,6 +28,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
     def _check_spec(self, spec):
         @jitclass(spec)
         class Test(object):
+
             def __init__(self):
                 pass
 
@@ -45,6 +56,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
         spec2 = [(1, int32), ('y', float32[:])]
 
         class Test(object):
+
             def __init__(self):
                 pass
 
@@ -65,6 +77,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
         @jitclass(spec)
         class Float2AndArray(object):
+
             def __init__(self, x, y, arr):
                 self.x = x
                 self.y = y
@@ -84,6 +97,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
         @jitclass(spec)
         class Vector2(object):
+
             def __init__(self, x, y):
                 self.x = x
                 self.y = y
@@ -127,20 +141,22 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
         arr = np.arange(10, dtype=np.float32)
         obj = Float2AndArray(1, 2, arr)
-        self.assertEqual(obj._meminfo.refcount, 1)
-        self.assertEqual(obj._meminfo.data, obj._dataptr)
+        obj_meminfo = _get_meminfo(obj)
+        self.assertEqual(obj_meminfo.refcount, 2)
+        self.assertEqual(obj_meminfo.data, _box.box_get_dataptr(obj))
         self.assertEqual(obj._numba_type_.class_type,
                          Float2AndArray.class_type)
         # Use jit class instance in numba
         other = identity(obj)
-        self.assertEqual(obj._meminfo.refcount, 2)
-        self.assertEqual(other._meminfo.refcount, 2)
-        self.assertEqual(other._meminfo.data, other._dataptr)
-        self.assertEqual(other._meminfo.data, obj._meminfo.data)
+        other_meminfo = _get_meminfo(other)  # duplicates MemInfo object to obj
+        self.assertEqual(obj_meminfo.refcount, 4)
+        self.assertEqual(other_meminfo.refcount, 4)
+        self.assertEqual(other_meminfo.data, _box.box_get_dataptr(other))
+        self.assertEqual(other_meminfo.data, obj_meminfo.data)
 
         # Check dtor
-        del other
-        self.assertEqual(obj._meminfo.refcount, 1)
+        del other, other_meminfo
+        self.assertEqual(obj_meminfo.refcount, 2)
 
         # Check attributes
         out_x, out_y, out_arr = retrieve_attributes(obj)
@@ -173,6 +189,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
         @jitclass(spec)
         class Foo(object):
+
             def __init__(self, val):
                 self.val = val
 
@@ -193,6 +210,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
         @jitclass(spec)
         class LinkedNode(object):
+
             def __init__(self, data, next):
                 self.data = data
                 self.next = next
@@ -208,19 +226,22 @@ class TestJitClass(TestCase, MemoryLeakMixin):
         self.assertIsNone(first.next)
 
         second = LinkedNode(321, first)
-        self.assertEqual(first._meminfo.refcount, 2)
+
+        first_meminfo = _get_meminfo(first)
+        second_meminfo = _get_meminfo(second)
+        self.assertEqual(first_meminfo.refcount, 3)
         self.assertEqual(second.next.data, first.data)
-        self.assertEqual(first._meminfo.refcount, 2)
-        self.assertEqual(second._meminfo.refcount, 1)
+        self.assertEqual(first_meminfo.refcount, 3)
+        self.assertEqual(second_meminfo.refcount, 2)
 
         # Test using deferred type as argument
         first_val = second.get_next_data()
         self.assertEqual(first_val, first.data)
 
         # Check ownership
-        self.assertEqual(first._meminfo.refcount, 2)
-        del second
-        self.assertEqual(first._meminfo.refcount, 1)
+        self.assertEqual(first_meminfo.refcount, 3)
+        del second, second_meminfo
+        self.assertEqual(first_meminfo.refcount, 2)
 
     def test_c_structure(self):
         spec = OrderedDict()
@@ -230,6 +251,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
         @jitclass(spec)
         class Struct(object):
+
             def __init__(self, a, b, c):
                 self.a = a
                 self.b = b
@@ -244,12 +266,26 @@ class TestJitClass(TestCase, MemoryLeakMixin):
                 ('c', ctypes.c_double),
             ]
 
-        ptr = ctypes.c_void_p(st._dataptr)
+        ptr = ctypes.c_void_p(_box.box_get_dataptr(st))
         cstruct = ctypes.cast(ptr, ctypes.POINTER(CStruct))[0]
         self.assertEqual(cstruct.a, st.a)
         self.assertEqual(cstruct.b, st.b)
         self.assertEqual(cstruct.c, st.c)
 
+    def test_is(self):
+        Vector = self._make_Vector2()
+        vec_a = Vector(1, 2)
+        vec_b = Vector(1, 2)
+
+        @njit
+        def do_is(a, b):
+            return a is b
+
+        with self.assertRaises(LoweringError) as raises:
+            # trigger compilation
+            do_is(vec_a, vec_a)
+        self.assertIn('no default `is` implementation', str(raises.exception))
+
     def test_isinstance(self):
         Vector2 = self._make_Vector2()
         vec = Vector2(1, 2)
@@ -265,11 +301,13 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
     def test_base_class(self):
         class Base(object):
+
             def what(self):
                 return self.attr
 
         @jitclass([('attr', int32)])
         class Test(Base):
+
             def __init__(self, attr):
                 self.attr = attr
 
@@ -294,6 +332,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
     def test_user_getter_setter(self):
         @jitclass([('attr', int32)])
         class Foo(object):
+
             def __init__(self, attr):
                 self.attr = attr
 
@@ -330,6 +369,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
     def test_user_deleter_error(self):
         class Foo(object):
+
             def __init__(self):
                 pass
 
@@ -348,6 +388,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
     def test_name_shadowing_error(self):
         class Foo(object):
+
             def __init__(self):
                 pass
 
@@ -370,6 +411,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
         # Different classes with the same names shouldn't confuse the compiler
         @jitclass([('x', int32)])
         class Foo(object):
+
             def __init__(self, x):
                 self.x = x + 2
 
@@ -380,6 +422,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
         @jitclass([('x', int32)])
         class Foo(object):
+
             def __init__(self, x):
                 self.x = x - 2
 
@@ -396,6 +439,7 @@ class TestJitClass(TestCase, MemoryLeakMixin):
 
     def test_parameterized(self):
         class MyClass(object):
+
             def __init__(self, value):
                 self.value = value
 
@@ -415,6 +459,95 @@ class TestJitClass(TestCase, MemoryLeakMixin):
         d = create_my_class(np.array([12.3]))
         np.testing.assert_equal(d.value, [12.3])
 
+    @tag('important')
+    def test_protected_attrs(self):
+        spec = {
+            'value': int32,
+            '_value': float32,
+            '__value': int32,
+            '__value__': int32,
+        }
+
+        @jitclass(spec)
+        class MyClass(object):
+
+            def __init__(self, value):
+                self.value = value
+                self._value = value / 2
+                self.__value = value * 2
+                self.__value__ = value - 1
+
+            @property
+            def private_value(self):
+                return self.__value
+
+            @property
+            def _inner_value(self):
+                return self._value
+
+            @_inner_value.setter
+            def _inner_value(self, v):
+                self._value = v
+
+            @property
+            def __private_value(self):
+                return self.__value
+
+            @__private_value.setter
+            def __private_value(self, v):
+                self.__value = v
+
+            def swap_private_value(self, new):
+                old = self.__private_value
+                self.__private_value = new
+                return old
+
+            def _protected_method(self, factor):
+                return self._value * factor
+
+            def __private_method(self, factor):
+                return self.__value * factor
+
+            def check_private_method(self, factor):
+                return self.__private_method(factor)
+
+
+        value = 123
+        inst = MyClass(value)
+        # test attributes
+        self.assertEqual(inst.value, value)
+        self.assertEqual(inst._value, value / 2)
+        self.assertEqual(inst.private_value, value * 2)
+        # test properties
+        self.assertEqual(inst._inner_value, inst._value)
+        freeze_inst_value = inst._value
+        inst._inner_value -= 1
+        self.assertEqual(inst._inner_value, freeze_inst_value - 1)
+
+        self.assertEqual(inst.swap_private_value(321), value * 2)
+        self.assertEqual(inst.swap_private_value(value * 2), 321)
+        # test methods
+        self.assertEqual(inst._protected_method(3), inst._value * 3)
+        self.assertEqual(inst.check_private_method(3), inst.private_value * 3)
+        # test special
+        self.assertEqual(inst.__value__, value - 1)
+        inst.__value__ -= 100
+        self.assertEqual(inst.__value__, value - 101)
+
+        # test errors
+        @njit
+        def access_dunder(inst):
+            return inst.__value
+
+        with self.assertRaises(errors.UntypedAttributeError) as raises:
+            access_dunder(inst)
+        # It will appear as "_TestJitClass__value" because the `access_dunder`
+        # is under the scope of 'TestJitClass'.
+        self.assertIn('_TestJitClass__value', str(raises.exception))
+
+        with self.assertRaises(AttributeError) as raises:
+            access_dunder.py_func(inst)
+        self.assertIn('_TestJitClass__value', str(raises.exception))
 
 if __name__ == '__main__':
     unittest.main()
