@@ -4,7 +4,7 @@ Testing object mode specifics.
 """
 from __future__ import print_function
 
-import numpy
+import numpy as np
 
 import numba.unittest_support as unittest
 from numba.compiler import compile_isolated, Flags
@@ -73,7 +73,7 @@ class TestObjectMode(TestCase):
 
     def test_array_of_object(self):
         cfunc = jit(array_of_object)
-        objarr = numpy.array([object()] * 10)
+        objarr = np.array([object()] * 10)
         self.assertIs(cfunc(objarr), objarr)
 
     def test_sequence_contains(self):
