@@ -31,6 +31,10 @@ def tuple_second(tup):
 def tuple_index(tup, idx):
     return tup[idx]
 
+def tuple_index_static(tup):
+    # Note the negative index
+    return tup[-2]
+
 def tuple_slice2(tup):
     return tup[1:-1]
 
@@ -61,6 +65,9 @@ def lt_usecase(a, b):
 def le_usecase(a, b):
     return a <= b
 
+def in_usecase(a, b):
+    return a in b
+
 def bool_usecase(tup):
     return bool(tup), (3 if tup else 2)
 
@@ -172,10 +179,30 @@ class TestOperations(TestCase):
         pyfunc = tuple_index
         cr = compile_isolated(pyfunc,
                               [types.UniTuple(types.int64, 3), types.int64])
-        tup = (4, 5, 6)
+        tup = (4, 3, 6)
         for i in range(len(tup)):
             self.assertPreciseEqual(cr.entry_point(tup, i), tup[i])
 
+        # With a compile-time static index (the code generation path is different)
+        pyfunc = tuple_index_static
+        for typ in (types.UniTuple(types.int64, 4),
+                    types.Tuple((types.int64, types.int32, types.int64, types.int32))):
+            cr = compile_isolated(pyfunc, (typ,))
+            tup = (4, 3, 42, 6)
+            self.assertPreciseEqual(cr.entry_point(tup), pyfunc(tup))
+
+        typ = types.UniTuple(types.int64, 1)
+        with self.assertTypingError():
+            cr = compile_isolated(pyfunc, (typ,))
+
+    def test_in(self):
+        pyfunc = in_usecase
+        cr = compile_isolated(pyfunc,
+                              [types.int64, types.UniTuple(types.int64, 3)])
+        tup = (4, 1, 5)
+        for i in range(5):
+            self.assertPreciseEqual(cr.entry_point(i, tup), pyfunc(i, tup))
+
     def check_slice(self, pyfunc):
         tup = (4, 5, 6, 7)
         cr = compile_isolated(pyfunc,
