@@ -938,6 +938,10 @@ class TestUFuncs(BaseUFuncTest, TestCase):
 
 class TestArrayOperators(BaseUFuncTest, TestCase):
 
+    def _check_results(self, expected, got):
+        self.assertEqual(expected.dtype.kind, got.dtype.kind)
+        np.testing.assert_array_almost_equal(expected, got)
+
     def unary_op_test(self, operator, flags=enable_nrt_flags,
                       skip_inputs=[], additional_inputs=[],
                       int_output_type=None, float_output_type=None):
@@ -956,8 +960,8 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
                                     flags=flags)
             cfunc = cr.entry_point
             expected = pyfunc(input_operand)
-            result = cfunc(input_operand)
-            np.testing.assert_array_almost_equal(expected, result)
+            got = cfunc(input_operand)
+            self._check_results(expected, got)
 
     def binary_op_test(self, operator, flags=enable_nrt_flags,
                        skip_inputs=[], additional_inputs=[],
@@ -996,22 +1000,31 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
                                     flags=flags)
             cfunc = cr.entry_point
             expected = pyfunc(input_operand0, input_operand1)
-            result = cfunc(input_operand0, input_operand1)
-            np.testing.assert_array_almost_equal(expected, result)
+            got = cfunc(input_operand0, input_operand1)
+            self._check_results(expected, got)
+
+    def bitwise_additional_inputs(self):
+        # For bitwise operators, we want to check the results for boolean
+        # arrays (see #1813).
+        return [
+            (True, types.boolean),
+            (False, types.boolean),
+            (np.array([True, False]), types.Array(types.boolean, 1, 'C')),
+            ]
 
     def binary_int_op_test(self, *args, **kws):
-        if 'skip_inputs' not in kws:
-            kws['skip_inputs'] = []
-        kws['skip_inputs'].extend([
+        skip_inputs = kws.setdefault('skip_inputs', [])
+        skip_inputs += [
             types.float32, types.float64,
             types.Array(types.float32, 1, 'C'),
-            types.Array(types.float64, 1, 'C')])
+            types.Array(types.float64, 1, 'C'),
+            ]
         return self.binary_op_test(*args, **kws)
 
-    def _make_arrays(self, dtypes):
-        for dtype in dtypes:
-            yield np.linspace(0, 5, 3).astype(dtype)
-            yield np.linspace(1, 6, 3).astype(dtype)
+    def binary_bitwise_op_test(self, *args, **kws):
+        additional_inputs = kws.setdefault('additional_inputs', [])
+        additional_inputs += self.bitwise_additional_inputs()
+        return self.binary_int_op_test(*args, **kws)
 
     def inplace_op_test(self, operator, lhs_values, rhs_values,
                         lhs_dtypes, rhs_dtypes):
@@ -1048,9 +1061,14 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
                                     (np.float32, np.float64, np.int64))
 
     def inplace_int_op_test(self, operator, lhs_values, rhs_values):
-        return self.inplace_op_test(operator, lhs_values, rhs_values,
-                                    (np.int16, np.int32, np.int64),
-                                    (np.int16, np.uint32))
+        self.inplace_op_test(operator, lhs_values, rhs_values,
+                             (np.int16, np.int32, np.int64),
+                             (np.int16, np.uint32))
+
+    def inplace_bitwise_op_test(self, operator, lhs_values, rhs_values):
+        self.inplace_int_op_test(operator, lhs_values, rhs_values)
+        self.inplace_op_test(operator, lhs_values, rhs_values,
+                             (np.bool_,), (np.bool_, np.bool_))
 
     # ____________________________________________________________
     # Unary operators
@@ -1063,10 +1081,11 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
         self.unary_op_test('-')
 
     def test_unary_invert_array_op(self):
-        self.unary_op_test('~', skip_inputs=[
-            types.float32, types.float64,
-            types.Array(types.float32, 1, 'C'),
-            types.Array(types.float64, 1, 'C')])
+        self.unary_op_test('~',
+                           skip_inputs=[types.float32, types.float64,
+                                        types.Array(types.float32, 1, 'C'),
+                                        types.Array(types.float64, 1, 'C')],
+                           additional_inputs=self.bitwise_additional_inputs())
 
     # ____________________________________________________________
     # Inplace operators
@@ -1084,7 +1103,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
         self.inplace_float_op_test('*=', [-1, 1.5, 3], [-5, 0, 2.5])
 
     def test_inplace_floordiv(self):
-        self.inplace_float_op_test('//=', [-1, 1.5, 3], [-5, 0, 2.5])
+        self.inplace_float_op_test('//=', [-1, 1.5, 3], [-5, 1.25, 2.5])
 
     def test_inplace_div(self):
         self.inplace_float_op_test('/=', [-1, 1.5, 3], [-5, 0, 2.5])
@@ -1097,13 +1116,13 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
         self.inplace_float_op_test('**=', [-1, 1.5, 3], [-5, 2, 2.5])
 
     def test_inplace_and(self):
-        self.inplace_int_op_test('&=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
+        self.inplace_bitwise_op_test('&=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
 
     def test_inplace_or(self):
-        self.inplace_int_op_test('|=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
+        self.inplace_bitwise_op_test('|=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
 
     def test_inplace_xor(self):
-        self.inplace_int_op_test('^=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
+        self.inplace_bitwise_op_test('^=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
 
     @tag('important')
     def test_inplace_lshift(self):
@@ -1158,6 +1177,29 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
 
     @tag('important')
     def test_floor_divide_array_op(self):
+        # Avoid floating-point zeros as x // 0.0 can have varying results
+        # depending on the algorithm (which changed accross Numpy versions)
+        self.inputs = [
+            (np.uint32(1), types.uint32),
+            (np.int32(-2), types.int32),
+            (np.int32(0), types.int32),
+            (np.uint64(4), types.uint64),
+            (np.int64(-5), types.int64),
+            (np.int64(0), types.int64),
+
+            (np.float32(-0.5), types.float32),
+            (np.float32(1.5), types.float32),
+
+            (np.float64(-2.5), types.float64),
+            (np.float64(3.5), types.float64),
+
+            (np.array([1,2], dtype='u4'), types.Array(types.uint32, 1, 'C')),
+            (np.array([3,4], dtype='u8'), types.Array(types.uint64, 1, 'C')),
+            (np.array([-1,1,5], dtype='i4'), types.Array(types.int32, 1, 'C')),
+            (np.array([-1,1,6], dtype='i8'), types.Array(types.int64, 1, 'C')),
+            (np.array([-0.5, 1.5], dtype='f4'), types.Array(types.float32, 1, 'C')),
+            (np.array([-2.5, 3.5], dtype='f8'), types.Array(types.float64, 1, 'C')),
+            ]
         self.binary_op_test('//')
 
     @tag('important')
@@ -1178,15 +1220,15 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
 
     @tag('important')
     def test_bitwise_and_array_op(self):
-        self.binary_int_op_test('&')
+        self.binary_bitwise_op_test('&')
 
     @tag('important')
     def test_bitwise_or_array_op(self):
-        self.binary_int_op_test('|')
+        self.binary_bitwise_op_test('|')
 
     @tag('important')
     def test_bitwise_xor_array_op(self):
-        self.binary_int_op_test('^')
+        self.binary_bitwise_op_test('^')
 
     @tag('important')
     def test_equal_array_op(self):
@@ -1438,7 +1480,8 @@ class _LoopTypesTester(TestCase):
 
         # Check each array (including inputs, to ensure they weren't
         # mutated).
-        for c_arg, py_arg in zip(c_args, py_args):
+        for dtype, py_arg, c_arg in zip(arg_dty, py_args, c_args):
+            py_arg, c_arg = self._fixup_results(dtype, py_arg, c_arg)
             typechar = c_arg.dtype.char
             ulps = self._ulps.get((ufunc.__name__, typechar), 1)
             prec = 'single' if typechar in 'fF' else 'exact'
@@ -1449,6 +1492,9 @@ class _LoopTypesTester(TestCase):
             self.assertPreciseEqual(py_arg, c_arg, prec=prec, msg=msg,
                                     ulps=ulps)
 
+    def _fixup_results(self, dtype, py_arg, c_arg):
+        return py_arg, c_arg
+
     @classmethod
     def _check_ufunc_loops(cls, ufunc):
         for loop in ufunc.types:
@@ -1578,11 +1624,33 @@ class TestLoopTypesIntRightShiftNoPython(_LoopTypesTester):
 TestLoopTypesIntRightShiftNoPython.autogenerate()
 
 
+class TestLoopTypesFloorDivideNoPython(_LoopTypesTester):
+    _compile_flags = no_pyobj_flags
+    _ufuncs = [np.floor_divide, np.remainder]
+    _required_types = 'bBhHiIlLqQfdFD'
+    _skip_types = 'mMO' + _LoopTypesTester._skip_types
+
+    def _fixup_results(self, dtype, py_arg, c_arg):
+        if dtype.kind == 'f':
+            # Discrepancies on floating-point floor division and remainder:
+            # Numpy may return nan where Numba returns inf, e.g. 1. // 0.
+            pred = (np.isinf(c_arg) & np.isnan(py_arg))
+            # Numpy and Numba may differ in signed zeros, e.g. -0. // -1.
+            pred |= (py_arg == 0.0) & (c_arg == 0.0)
+            c_arg[pred] = py_arg[pred]
+        return py_arg, c_arg
+
+TestLoopTypesFloorDivideNoPython.autogenerate()
+
+
 class TestLoopTypesFloatNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = supported_ufuncs[:]
     if iswindows:
         _ufuncs.remove(np.signbit) # TODO: fix issue #758
+    _ufuncs.remove(np.floor_divide) # has its own test class
+    _ufuncs.remove(np.remainder) # has its own test class
+    _ufuncs.remove(np.mod) # same as np.remainder
     _required_types = 'fd'
     _skip_types = 'FDmMO' + _LoopTypesTester._skip_types
 
@@ -1732,6 +1800,5 @@ class TestUFuncBadArgsNoPython(TestCase):
                           return_type=types.float64, flags=self._compile_flags)
 
 
-
 if __name__ == '__main__':
     unittest.main()
