@@ -2,7 +2,7 @@ from __future__ import print_function, division, absolute_import
 
 import itertools
 
-import numpy
+import numpy as np
 
 from numba import types
 
@@ -246,7 +246,8 @@ class BitwiseRightShift(BitwiseShiftOperation):
 
 
 class BitwiseLogicOperation(BinOp):
-    cases = list(integer_binop_cases)
+    cases = [signature(types.boolean, types.boolean, types.boolean)]
+    cases += list(integer_binop_cases)
 
 
 @infer
@@ -272,7 +273,10 @@ class BitwiseXor(BitwiseLogicOperation):
 class BitwiseInvert(ConcreteTemplate):
     key = "~"
 
-    cases = [signature(types.int8, types.boolean)]
+    # Note Numba follows the Numpy semantics of returning a bool,
+    # while Python returns an int.  This makes it consistent with
+    # np.invert() and makes array expressions correct.
+    cases = [signature(types.boolean, types.boolean)]
     cases += [signature(choose_result_int(op), op) for op in types.unsigned_domain]
     cases += [signature(choose_result_int(op), op) for op in types.signed_domain]
 
@@ -614,7 +618,7 @@ class NumberClassAttribute(AttributeTemplate):
             if isinstance(val, (types.BaseTuple, types.Sequence)):
                 # Array constructor, e.g. np.int32([1, 2])
                 sig = self.context.resolve_function_type(
-                    numpy.array, (val,), {'dtype': types.DType(ty)})
+                    np.array, (val,), {'dtype': types.DType(ty)})
                 return sig.return_type
             else:
                 # Scalar constructor, e.g. np.int32(42)
