@@ -12,8 +12,7 @@ from . import templates
 from .typeof import typeof_impl
 
 
-CTYPES_MAP = {
-    None: types.none,
+_FROM_CTYPES = {
     ctypes.c_bool: types.boolean,
     
     ctypes.c_int8:  types.int8,
@@ -33,12 +32,55 @@ CTYPES_MAP = {
     ctypes.py_object: types.ffi_forced_object,
 }
 
+_TO_CTYPES = {v: k for (k, v) in _FROM_CTYPES.items()}
 
-def convert_ctypes(ctypeobj):
-    try:
-        return CTYPES_MAP[ctypeobj]
-    except KeyError:
-        raise TypeError("unhandled ctypes type: %s" % ctypeobj)
+
+def from_ctypes(ctypeobj):
+    """
+    Convert the given ctypes type to a Numba type.
+    """
+    if ctypeobj is None:
+        # Special case for the restype of void-returning functions
+        return types.none
+
+    assert isinstance(ctypeobj, type), ctypeobj
+
+    def _convert_internal(ctypeobj):
+        # Recursive helper
+        if issubclass(ctypeobj, ctypes._Pointer):
+            valuety = _convert_internal(ctypeobj._type_)
+            if valuety is not None:
+                return types.CPointer(valuety)
+        else:
+            return _FROM_CTYPES.get(ctypeobj)
+
+    ty = _convert_internal(ctypeobj)
+    if ty is None:
+        raise TypeError("Unsupported ctypes type: %s" % ctypeobj)
+    return ty
+
+
+def to_ctypes(ty):
+    """
+    Convert the given Numba type to a ctypes type.
+    """
+    assert isinstance(ty, types.Type), ty
+
+    if ty is types.none:
+        # Special case for the restype of void-returning functions
+        return None
+
+    def _convert_internal(ty):
+        if isinstance(ty, types.CPointer):
+            return ctypes.POINTER(_convert_internal(ty.dtype))
+        else:
+            return _TO_CTYPES.get(ty)
+
+    ctypeobj = _convert_internal(ty)
+    if ctypeobj is None:
+        raise TypeError("Cannot convert Numba type '%s' to ctypes type"
+                        % (ty,))
+    return ctypeobj
 
 
 def is_ctypes_funcptr(obj):
@@ -68,9 +110,9 @@ def make_function_type(cfnptr):
         raise TypeError("ctypes function %r doesn't define its argument types; "
                         "consider setting the `argtypes` attribute"
                         % (cfnptr.__name__,))
-    cargs = [convert_ctypes(a)
+    cargs = [from_ctypes(a)
              for a in cfnptr.argtypes]
-    cret = convert_ctypes(cfnptr.restype)
+    cret = from_ctypes(cfnptr.restype)
     if sys.platform == 'win32' and not cfnptr._flags_ & ctypes._FUNCFLAG_CDECL:
         # 'stdcall' calling convention under Windows
         cconv = 'x86_stdcallcc'
