@@ -11,7 +11,7 @@ import timeit
 import math
 import sys
 
-import numpy
+import numpy as np
 
 from .six import *
 from numba.config import PYVERSION, MACHINE_BITS
@@ -57,10 +57,42 @@ try:
     from functools import singledispatch
 except ImportError:
     try:
-        from singledispatch import singledispatch
+        import singledispatch
     except ImportError:
         raise ImportError("please install the 'singledispatch' package "
                           "('pip install singledispatch')")
+    else:
+        # Hotfix for https://bitbucket.org/ambv/singledispatch/issues/8/inconsistent-hierarchy-with-enum
+        def _c3_merge(sequences):
+            """Merges MROs in *sequences* to a single MRO using the C3 algorithm.
+
+            Adapted from http://www.python.org/download/releases/2.3/mro/.
+
+            """
+            result = []
+            while True:
+                sequences = [s for s in sequences if s]   # purge empty sequences
+                if not sequences:
+                    return result
+                for s1 in sequences:   # find merge candidates among seq heads
+                    candidate = s1[0]
+                    for s2 in sequences:
+                        if candidate in s2[1:]:
+                            candidate = None
+                            break      # reject the current head, it appears later
+                    else:
+                        break
+                if candidate is None:
+                    raise RuntimeError("Inconsistent hierarchy")
+                result.append(candidate)
+                # remove the chosen candidate
+                for seq in sequences:
+                    if seq[0] == candidate:
+                        del seq[0]
+
+        singledispatch._c3_merge = _c3_merge
+
+        singledispatch = singledispatch.singledispatch
 
 
 # Mapping between operator module functions and the corresponding built-in
@@ -297,8 +329,8 @@ class BenchmarkResult(object):
     def __init__(self, func, records, loop):
         self.func = func
         self.loop = loop
-        self.records = numpy.array(records) / loop
-        self.best = numpy.min(self.records)
+        self.records = np.array(records) / loop
+        self.best = np.min(self.records)
 
     def __repr__(self):
         name = getattr(self.func, "__name__", self.func)
